@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package Mojo::Home;
 
@@ -12,8 +12,8 @@ use File::Spec;
 use FindBin;
 use Mojo::Script;
 
-__PACKAGE__->attr(app_class => (chained => 1));
-__PACKAGE__->attr(parts => (chained => 1, default => sub { [] }));
+__PACKAGE__->attr('app_class');
+__PACKAGE__->attr(parts => (default => sub { [] }));
 
 # I'm normally not a praying man, but if you're up there,
 # please save me Superman.
@@ -56,7 +56,10 @@ sub detect {
             my @home = File::Spec->splitdir($path);
 
             # Remove "lib" and "blib"
-            pop @home while $home[-1] =~ /^b?lib$/ || $home[-1] eq '';
+            while (@home) {
+                last unless $home[-1] =~ /^b?lib$/ || $home[-1] eq '';
+                pop @home;
+            }
 
             # Check for executable
             return $self->parts(\@home)
