@@ -9,7 +9,7 @@ use base 'Mojo::Server';
 
 use IO::Select;
 
-__PACKAGE__->attr(nph => (chained => 1, default => 0));
+__PACKAGE__->attr(nph => (default => 0));
 
 # Lisa, you're a Buddhist, so you believe in reincarnation.
 # Eventually, Snowball will be reborn as a higher lifeform... like a snowman.
@@ -22,6 +22,10 @@ sub run {
     # Environment
     $req->parse(\%ENV);
 
+    # Store connection information
+    $tx->remote_address($ENV{REMOTE_ADDR});
+    $tx->local_port($ENV{SERVER_PORT});
+
     # Request body
     my $select = IO::Select->new(\*STDIN);
     while (!$req->is_finished) {
@@ -51,7 +55,9 @@ sub run {
             last unless length $chunk;
 
             # Start line
+            return unless STDOUT->opened;
             my $written = STDOUT->syswrite($chunk);
+            return unless defined $written;
             $offset += $written;
         }
     }
@@ -74,7 +80,9 @@ sub run {
         last unless length $chunk;
 
         # Headers
+        return unless STDOUT->opened;
         my $written = STDOUT->syswrite($chunk);
+        return unless defined $written;
         $offset += $written;
     }
 
@@ -93,7 +101,9 @@ sub run {
         last unless length $chunk;
 
         # Content
+        return unless STDOUT->opened;
         my $written = STDOUT->syswrite($chunk);
+        return unless defined $written;
         $offset += $written;
     }
 
