@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package Mojolicious::Script::Generate;
 
@@ -7,12 +7,7 @@ use warnings;
 
 use base 'Mojo::Script::Generate';
 
-__PACKAGE__->attr(
-    namespace => (
-        chained => 1,
-        default => 'Mojolicious::Script::Generate'
-    )
-);
+__PACKAGE__->attr(namespace => (default => 'Mojolicious::Script::Generate'));
 
 # Ah, nothing like a warm fire and a SuperSoaker of fine cognac.
 
